import os
import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import ant_impedance
import days_of_year
import binned_values
import moving_average

#----------- plot Q for Y + In vs Out plasmasphere -----------------
def plot_Q_In_Out(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, fl):

    if len(fl) == 0:
        print('Read calculated antenna parameters ...')
        fdir=fpath.strip()
        inpflag=inpflag.strip()
        if fdir[len(fdir)-1] == '/':
            datafile=fdir+'All-Bands_InOut'+'_data.dat'
        else:
            datafile=fdir+'/All-Bands_InOut'+'_data.dat'

        with open(datafile,'r') as dataf:
            dataf.readline()
            dataf.readline()

            fl=dataf.readlines()

            dataf.close()

        print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,2.0),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.14,0.55]
    yB=[0.50,0.50]
    wh=[0.38,0.38]
    ht=[0.35,0.35]

    axes=[]

    for i in range(2):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(2):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=bname
    sL1=sL1+' Y+ TATU 1st Harmonic'

    minQ=0
    maxQ=60

    date=[]
    fres=[[],[]]
    Q=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        date.append(datet)
        L1=float(a[3])
        inpl=int(a[24])

        if bname != 'All-Bands':
            if L1 != L:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L) \
                    +' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpl == 1:
            fq1=float(a[5])

            fres[0].append(fq1/1.0e3)         #frequency in kHz
            Q[0].append(float(a[7]))
        else:
            fq2=float(a[5])
            
            fres[1].append(fq2/1.0e3)
            Q[1].append(float(a[7]))

    #end 'for loop' for reading data file

    if diff > 0:
        print(diff,'data out of',nrecd,'not used')

    fsmin=[min(fres[0]), min(fres[1])]
    fsmax=[max(fres[0]), max(fres[1])]

    if bname == 'All-Bands':
        xscale='log'
        fmin=2.5
        fmax=40
    else:
        xscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    N=6   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=50 # for 2D binning not for 1D
    y=np.zeros((ybin+1))
    yQ=np.zeros(ybin)

    for i in range(2):
        xbin=int((fsmax[i]-fsmin[i])/bwd)
        x=[0]*(xbin+1)
        xfr=[0]*xbin
        for k in range(xbin+1):
            x[k]=fsmin[i]+float(k)*bwd
            if k > 0:
                xfr[k-1]=0.5*(x[k-1]+x[k])

        #--- determine y min & max
        Qmin=min(Q[i])
        Qmax=max(Q[i])
        dQ=(Qmax-Qmin)/ybin

        #--- bin size and center of bins for Ca, Ra and pwr
        for k in range(ybin+1):
            y[k]=Qmin+float(k)*dQ
            if k > 0:
                yQ[k-1]=0.5*([k-1]+y[k])

        # --- calculate number of Ca data points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],Q[i],fres[i],statistic='count', \
            bins=[x,y])
        a=counts.statistic  # number of data points for Ca
        a=a.transpose()

        rdl=len(a[0])+1-N
        Qb2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            Qb2d[k]=moving_average.data_points_running_average(a[k], N)
        Qb2d=Qb2d.transpose()

        # plot number of data points in each bin, onece for yQ
        # points at given frequency xfr[j]. The data points number is normalized
        # by maximum at each frequency
        for j in range(len(xfr[0:rdl])):
            xx=[xfr[j]]*ybin
            Qbmax=max(Qb2d[j])
            if Qbmax > 0:
                sc=axes[i].scatter(xx,yQ,c=Qb2d[j]/Qbmax,cmap=colormp,s=0.1, \
                    marker='s')

        #----- set axis properties for Ca ------
        axes[i].set_xlim(fmin,fmax)
        axes[i].set_xscale(xscale)
        axes[i].set_ylim(minQ,maxQ)
        axes[i].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.2,left=True,right=True,bottom=True,top=True, \
            direction='out')
        axes[i].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='out')
        axes[i].yaxis.set_minor_locator(AutoMinorLocator())

        # --- y-axis label
        axes[i].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5,labelpad=0.7)
        if bname == 'All-Bands':
            axes[i].set_xticks([2.5,5,10,20,30,40])
            axes[i].set_xticklabels(['2.5','5','10','20','30','40'])

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)

    axes[0].set_ylabel('$Q$-Factor',fontsize=2.5,labelpad=0.7)
    axes[1].set_yticklabels([])
    axes[0].text(1,1.12,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(1,1.04,sL1,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)
    axes[0].text(0,1.04,'Inside Plasmasphere',fontsize=2.5,ha='left',va='bottom', \
        transform=axes[0].transAxes)
    axes[1].text(1,1.04,'Outside Plasmasphere',fontsize=2.5,ha='right',va='bottom', \
        transform=axes[1].transAxes)

    # ---- color bar ----
    axins = inset_axes(axes[0], width='70%', height='7%', loc='lower left', \
    bbox_to_anchor=(0.61,-0.36,1,1), bbox_transform=axes[0].transAxes)
    cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar, fl

#----------- plot Q for Y + and Y- for both in and out plasmasphere -----------------
def plot_Q_all(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, fl):

    if len(fl) == 0:
        print('Read calculated antenna parameters ...')
        fdir=fpath.strip()
        inpflag=inpflag.strip()
        if fdir[len(fdir)-1] == '/':
            datafile=fdir+'All-Bands_InOut'+'_data.dat'
        else:
            datafile=fdir+'/All-Bands_InOut'+'_data.dat'

        with open(datafile,'r') as dataf:
            dataf.readline()
            dataf.readline()

            fl=dataf.readlines()

            dataf.close()

        print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,2.0),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.14,0.55]
    yB=[0.50,0.50]
    wh=[0.38,0.38]
    ht=[0.35,0.35]

    axes=[]

    for i in range(2):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(2):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=bname + ' and All Regions'

    minQ=0
    maxQ=60

    date=[]
    fres=[[],[]]
    Q=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        date.append(datet)
        L1=float(a[3])

        if bname != 'All-Bands':
            if L1 != L:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L) \
                    +' '+datet+' '+a[1])
                diff=diff+1
                continue

        fq1=float(a[5])
        fres[0].append(fq1/1.0e3)         #frequency in kHz
        Q[0].append(float(a[7]))

        fq2=float(a[16])
        fres[1].append(fq2/1.0e3)
        Q[1].append(float(a[18]))

    #end 'for loop' for reading data file

    if diff > 0:
        print(diff,'data out of',nrecd,'not used')

    fsmin=[min(fres[0]), min(fres[1])]
    fsmax=[max(fres[0]), max(fres[1])]

    if bname == 'All-Bands':
        xscale='log'
        fmin=2.5
        fmax=40
    else:
        xscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    N=5   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=50 # for 2D binning not for 1D
    y=np.zeros((ybin+1))
    yQ=np.zeros(ybin)

    for i in range(2):
        xbin=int((fsmax[i]-fsmin[i])/bwd)
        x=[0]*(xbin+1)
        xfr=[0]*xbin
        for k in range(xbin+1):
            x[k]=fsmin[i]+float(k)*bwd
            if k > 0:
                xfr[k-1]=0.5*(x[k-1]+x[k])

        #--- determine y min & max
        Qmin=min(Q[i])
        Qmax=max(Q[i])
        dQ=(Qmax-Qmin)/ybin

        #--- bin size and center of bins for Ca, Ra and pwr
        for k in range(ybin+1):
            y[k]=Qmin+float(k)*dQ
            if k > 0:
                yQ[k-1]=0.5*([k-1]+y[k])

        # --- calculate number of Ca data points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],Q[i],fres[i],statistic='count', \
            bins=[x,y])
        a=counts.statistic  # number of data points for Ca
        a=a.transpose()

        rdl=len(a[0])+1-N
        Qb2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            Qb2d[k]=moving_average.data_points_running_average(a[k], N)
        Qb2d=Qb2d.transpose()

        # plot number of data points in each bin, onece for yQ
        # points at given frequency xfr[j]. The data points number is normalized
        # by maximum at each frequency
        for j in range(len(xfr[0:rdl])):
            xx=[xfr[j]]*ybin
            Qbmax=max(Qb2d[j])
            if Qbmax > 0:
                sc=axes[i].scatter(xx,yQ,c=Qb2d[j]/Qbmax,cmap=colormp,s=0.1, \
                    marker='s')

        #----- set axis properties for Ca ------
        axes[i].set_xlim(fmin,fmax)
        axes[i].set_xscale(xscale)
        axes[i].set_ylim(minQ,maxQ)
        axes[i].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.2,left=True,right=True,bottom=True,top=True, \
            direction='out')
        axes[i].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='out')
        axes[i].yaxis.set_minor_locator(AutoMinorLocator())
        axes[i].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5,labelpad=0.7)
        if bname == 'All-Bands':
            axes[i].set_xticks([2.5,5,10,20,30,40])
            axes[i].set_xticklabels(['2.5','5','10','20','30','40'])

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)

    # --- y-axis label
    axes[0].set_ylabel('$Q$-Factor',fontsize=2.5,labelpad=0.7)
    axes[1].set_yticklabels([])
    axes[0].text(1,1.12,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(1,1.04,sL1,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)
    axes[0].text(0,1.04,'Y+ 1st Harmonic',fontsize=2.5,ha='left',va='bottom', \
        transform=axes[0].transAxes)
    axes[1].text(1,1.04,'Y- 1st Harmonic',fontsize=2.5,ha='right',va='bottom', \
        transform=axes[1].transAxes)

    # ---- color bar ----
    axins = inset_axes(axes[0], width='70%', height='7%', loc='lower left', \
    bbox_to_anchor=(0.61,-0.36,1,1), bbox_transform=axes[0].transAxes)
    cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar, fl